@@ -183,11 +183,11 @@ static model_filter_init_t model_filter_init[2] = {
     // The dynamic analog range of one voice is approximately 1.5V,
     1.5,
     // riding at a DC level of approximately 5.0V.
-    5.075,
+    5.075,      // 5V +1.5%
     // Capacitor value.
     470e-12,
     // Transistor parameters.
-    12.18,
+    12.18,      // 12V +1.5%
     1.31,
     26.0e-3,
     1.0,
@@ -205,12 +205,11 @@ static model_filter_init_t model_filter_init[2] = {
     sizeof(opamp_voltage_8580)/sizeof(*opamp_voltage_8580),
     // FIXME: Measure for the 8580.
     0.24,
-    // FIXME: Measure for the 8580.
-    4.84,
+    4.7975,     // 4.75V +1%
     // Capacitor value.
     22e-9,
     // Transistor parameters.
-    9.09,
+    9.09,       // 9V +1%
     0.80,
     26.0e-3,
     1.0,   // Unused, leave at 1
@@ -244,7 +243,7 @@ static double log1p(double x)
 Filter::model_filter_t Filter::model_filter[2];
 
 // The 4.75V voltage for the virtual ground is generated by a PolySi resistor divider
-static const double Vref = 4.75;
+static const double Vref = 4.7975; // 4.75V +1%
 
 // ----------------------------------------------------------------------------
 // Constructor.
@@ -280,6 +279,12 @@ Filter::Filter()
       double N31 = norm*((1u << 31) - 1);
       mf.vo_N16 = N16;
 
+      // In the 6581 the mixer input resistors for the filter lines
+      // are slightly bigger than the voice ones
+      // Scale the values accordingly
+      const double scaleFactor = m==0 ? 0.93 : 1.0;
+      mf.filterGain = static_cast<int>(scaleFactor * (1 << 12));
+
       // The "zero" output level of the voices.
       // The digital range of one voice is 20 bits; create a scaling term
       // for multiplication which fits in 11 bits.
@@ -300,6 +305,8 @@ Filter::Filter()
       // double_point scaled_voltage[fi.opamp_voltage_size];
       double_point scaled_voltage[50];
 
+      assert((fi.opamp_voltage_size > 0) && (fi.opamp_voltage_size < 50));
+
       for (int i = 0; i < fi.opamp_voltage_size; i++) {
         // The target output range is 16 bits, in order to fit in an unsigned
         // short.
