@@ -1,7 +1,7 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2020 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2024 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
  * Copyright 2004 Dag Lem <resid@nimrod.no>
  *
@@ -22,46 +22,36 @@
 
 #include "SincResampler.h"
 
+#ifdef HAVE_CXX20
+#  include <numbers>
+#endif
+
+#include <algorithm>
+#include <iterator>
 #include <cassert>
 #include <cstring>
 #include <cmath>
-#include <iostream>
-#include <sstream>
+#include <cstdint>
 
 #include "siddefs-fp.h"
 
-#include "sidcxx11.h"
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#ifdef HAVE_EMMINTRIN_H
-#  include <emmintrin.h>
-#elif defined HAVE_MMINTRIN_H
-#  include <mmintrin.h>
+#ifdef HAVE_SMMINTRIN_H
+#  include <smmintrin.h>
 #elif defined(HAVE_ARM_NEON_H)
 #  include <arm_neon.h>
 #endif
-#ifdef HAVE_CXX11
-#  include <mutex>
-#endif
 
 namespace reSIDfp
 {
 
-typedef std::map<std::string, matrix_t> fir_cache_t;
-
-/// Cache for the expensive FIR table computation results.
-fir_cache_t FIR_CACHE;
-#ifdef HAVE_CXX11
-std::mutex FIR_CACHE_Lock;
-#endif
-
 /// Maximum error acceptable in I0 is 1e-6, or ~96 dB.
-const double I0E = 1e-6;
+constexpr double I0E = 1e-6;
 
-const int BITS = 16;
+constexpr int BITS = 16;
 
 /**
  * Compute the 0th order modified Bessel function of the first kind.
@@ -98,64 +88,56 @@ double I0(double x)
  * @param bLength length of the sinc buffer
  * @return convolved result
  */
-int convolve(const short* a, const short* b, int bLength)
+int convolve(const int* a, const short* b, int bLength)
 {
-#ifdef HAVE_EMMINTRIN_H
+#ifdef HAVE_SMMINTRIN_H
     int out = 0;
 
-    const uintptr_t offset = (uintptr_t)(a) & 0x0f;
+    const uintptr_t offset = (uintptr_t)(b) & 0x0f;
 
     // check for aligned accesses
-    if (offset == ((uintptr_t)(b) & 0x0f))
+    if (offset)
     {
-        if (offset)
+        const int l = (0x10 - offset) / 2;
+
+        for (int i = 0; i < l; i++)
         {
-            const int l = (0x10 - offset)/2;
-
-            for (int i = 0; i < l; i++)
-            {
-                out += *a++ * *b++;
-            }
-
-            bLength -= offset;
+            out += a[i] * static_cast<int>(b[i]);
         }
 
-        __m128i acc = _mm_setzero_si128();
-
-        const int n = bLength / 8;
-
-        for (int i = 0; i < n; i++)
-        {
-            const __m128i tmp = _mm_madd_epi16(*(__m128i*)a, *(__m128i*)b);
-            acc = _mm_add_epi32(acc, tmp);
-            a += 8;
-            b += 8;
-        }
-
-        __m128i vsum = _mm_add_epi32(acc, _mm_srli_si128(acc, 8));
-        vsum = _mm_add_epi32(vsum, _mm_srli_si128(vsum, 4));
-        out += _mm_cvtsi128_si32(vsum);
-
-        bLength &= 7;
+        a += l;
+        b += l;
+        bLength -= l;
     }
-#elif defined HAVE_MMINTRIN_H
-    __m64 acc = _mm_setzero_si64();
 
-    const int n = bLength / 4;
+    __m128i acc = _mm_setzero_si128();
+
+    const int n = bLength / 8;
 
     for (int i = 0; i < n; i++)
     {
-        const __m64 tmp = _mm_madd_pi16(*(__m64*)a, *(__m64*)b);
-        acc = _mm_add_pi16(acc, tmp);
+        const __m128i tmp_b = _mm_stream_load_si128((__m128i*)b);
+
+        __m128i val_b = _mm_cvtepi16_epi32(tmp_b);
+        __m128i prod = _mm_mullo_epi32(*(__m128i*)a, val_b);
+        acc = _mm_add_epi32(acc, prod);
         a += 4;
-        b += 4;
+
+        val_b = _mm_cvtepi16_epi32(_mm_srli_si128(tmp_b, 8));
+        prod = _mm_mullo_epi32(*(__m128i*)a, val_b);
+        acc = _mm_add_epi32(acc, prod);
+        a += 4;
+
+        b += 8;
     }
 
-    int out = _mm_cvtsi64_si32(acc) + _mm_cvtsi64_si32(_mm_srli_si64(acc, 32));
-    _mm_empty();
+    __m128i vsum = _mm_add_epi32(acc, _mm_srli_si128(acc, 8));
+    vsum = _mm_add_epi32(vsum, _mm_srli_si128(vsum, 4));
+    out += _mm_cvtsi128_si32(vsum);
 
-    bLength &= 3;
-#elif defined(HAVE_ARM_NEON_H)
+    bLength &= 7;
+
+/*#elif defined(HAVE_ARM_NEON_H)
 #if (defined(__arm64__) && defined(__APPLE__)) || defined(__aarch64__)
     int32x4_t acc1Low = vdupq_n_s32(0);
     int32x4_t acc1High = vdupq_n_s32(0);
@@ -216,9 +198,9 @@ int convolve(const short* a, const short* b, int bLength)
     bLength &= 3;
 #else
     int32x4_t acc = vdupq_n_s32(0);
-    
+
     const int n = bLength / 4;
-    
+
     for (int i = 0; i < n; i++)
     {
         const int16x4_t h_vec = vld1_s16(a);
@@ -227,21 +209,21 @@ int convolve(const short* a, const short* b, int bLength)
         a += 4;
         b += 4;
     }
-    
+
     int out = vgetq_lane_s32(acc, 0) +
               vgetq_lane_s32(acc, 1) +
               vgetq_lane_s32(acc, 2) +
               vgetq_lane_s32(acc, 3);
-    
+
     bLength &= 3;
-#endif
+#endif*/
 #else
     int out = 0;
 #endif
 
     for (int i = 0; i < bLength; i++)
     {
-        out += *a++ * *b++;
+        out += a[i] * static_cast<int>(b[i]);
     }
 
     return (out + (1 << 14)) >> 15;
@@ -273,17 +255,27 @@ int SincResampler::fir(int subcycle)
     return v1 + (firTableOffset * (v2 - v1) >> 10);
 }
 
-SincResampler::SincResampler(double clockFrequency, double samplingFrequency, double highestAccurateFrequency) :
-    sampleIndex(0),
-    cyclesPerSample(static_cast<int>(clockFrequency / samplingFrequency * 1024.)),
-    sampleOffset(0),
-    outputValue(0)
+SincResampler::SincResampler(
+        double clockFrequency,
+        double samplingFrequency,
+        double highestAccurateFrequency) :
+    cyclesPerSample(static_cast<int>(clockFrequency / samplingFrequency * 1024.))
 {
+#if defined(HAVE_CXX20) && defined(__cpp_lib_constexpr_cmath)
+    constexpr double PI = std::numbers::pi;
+#else
+#  ifdef M_PI
+        constexpr double PI = M_PI;
+#else
+        constexpr double PI = 3.14159265358979323846;
+#  endif
+#endif
+
     // 16 bits -> -96dB stopband attenuation.
-    const double A = -20. * log10(1.0 / (1 << BITS));
+    const double A = -20. * std::log10(1.0 / (1 << BITS));
     // A fraction of the bandwidth is allocated to the transition band, which we double
     // because we design the filter to transition halfway at nyquist.
-    const double dw = (1. - 2.*highestAccurateFrequency / samplingFrequency) * M_PI * 2.;
+    const double dw = (1. - 2.*highestAccurateFrequency / samplingFrequency) * PI * 2.;
 
     // For calculation of beta and N see the reference for the kaiserord
     // function in the MATLAB Signal Processing Toolbox:
@@ -291,6 +283,7 @@ SincResampler::SincResampler(double clockFrequency, double samplingFrequency, do
     const double beta = 0.1102 * (A - 8.7);
     const double I0beta = I0(beta);
     const double cyclesPerSampleD = clockFrequency / samplingFrequency;
+    const double inv_cyclesPerSampleD = samplingFrequency / clockFrequency;
 
     {
         // The filter order will maximally be 124 with the current constraints.
@@ -310,45 +303,22 @@ SincResampler::SincResampler(double clockFrequency, double samplingFrequency, do
         assert(firN < RINGSIZE);
 
         // Error is bounded by err < 1.234 / L^2, so L = sqrt(1.234 / (2^-16)) = sqrt(1.234 * 2^16).
-        firRES = static_cast<int>(ceil(sqrt(1.234 * (1 << BITS)) / cyclesPerSampleD));
+        firRES = static_cast<int>(std::ceil(std::sqrt(1.234 * (1 << BITS)) * inv_cyclesPerSampleD));
 
         // firN*firRES represent the total resolution of the sinc sampling. JOS
         // recommends a length of 2^BITS, but we don't quite use that good a filter.
         // The filter test program indicates that the filter performs well, though.
     }
 
-    // Create the map key
-    std::ostringstream o;
-    o << firN << "," << firRES << "," << cyclesPerSampleD;
-    const std::string firKey = o.str();
-
-#ifdef HAVE_CXX11
-    std::lock_guard<std::mutex> lock(FIR_CACHE_Lock);
-#endif
-
-    fir_cache_t::iterator lb = FIR_CACHE.lower_bound(firKey);
-
-    // The FIR computation is expensive and we set sampling parameters often, but
-    // from a very small set of choices. Thus, caching is used to speed initialization.
-    if (lb != FIR_CACHE.end() && !(FIR_CACHE.key_comp()(firKey, lb->first)))
-    {
-        firTable = &(lb->second);
-    }
-    else
     {
         // Allocate memory for FIR tables.
-        matrix_t tempTable(firRES, firN);
-#ifdef HAVE_CXX11
-        firTable = &(FIR_CACHE.emplace_hint(lb, fir_cache_t::value_type(firKey, tempTable))->second);
-#else
-        firTable = &(FIR_CACHE.insert(lb, fir_cache_t::value_type(firKey, tempTable))->second);
-#endif
+        firTable = new matrix_t(firRES, firN);
 
         // The cutoff frequency is midway through the transition band, in effect the same as nyquist.
-        const double wc = M_PI;
+        const double wc = PI;
 
         // Calculate the sinc tables.
-        const double scale = 32768.0 * wc / cyclesPerSampleD / M_PI;
+        const double scale = 32768.0 * wc * inv_cyclesPerSampleD / PI;
 
         // we're not interested in the fractional part
         // so use int division before converting to double
@@ -364,10 +334,10 @@ SincResampler::SincResampler(double clockFrequency, double samplingFrequency, do
                 const double x = j - jPhase;
 
                 const double xt = x / firN_2;
-                const double kaiserXt = fabs(xt) < 1. ? I0(beta * sqrt(1. - xt * xt)) / I0beta : 0.;
+                const double kaiserXt = std::fabs(xt) < 1. ? I0(beta * std::sqrt(1. - xt * xt)) / I0beta : 0.;
 
-                const double wt = wc * x / cyclesPerSampleD;
-                const double sincWt = fabs(wt) >= 1e-8 ? sin(wt) / wt : 1.;
+                const double wt = wc * x * inv_cyclesPerSampleD;
+                const double sincWt = std::fabs(wt) >= 1e-8 ? std::sin(wt) / wt : 1.;
 
                 (*firTable)[i][j] = static_cast<short>(scale * sincWt * kaiserXt);
             }
@@ -375,18 +345,16 @@ SincResampler::SincResampler(double clockFrequency, double samplingFrequency, do
     }
 }
 
+SincResampler::~SincResampler()
+{
+    delete firTable;
+}
+
 bool SincResampler::input(int input)
 {
     bool ready = false;
 
-    /*
-     * Clip the input as it may overflow the 16 bit range.
-     *
-     * Approximate measured input ranges:
-     * 6581: [-24262,+25080]  (Kawasaki_Synthesizer_Demo)
-     * 8580: [-21514,+35232]  (64_Forever, Drum_Fool)
-     */
-    sample[sampleIndex] = sample[sampleIndex + RINGSIZE] = softClip(input);
+    sample[sampleIndex] = sample[sampleIndex + RINGSIZE] = input;
     sampleIndex = (sampleIndex + 1) & (RINGSIZE - 1);
 
     if (sampleOffset < 1024)
@@ -403,7 +371,7 @@ bool SincResampler::input(int input)
 
 void SincResampler::reset()
 {
-    memset(sample, 0, sizeof(sample));
+    std::fill(std::begin(sample), std::end(sample), 0);
     sampleOffset = 0;
 }
 
