@@ -1,7 +1,7 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2013-2021 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2013-2024 Leandro Nini <drfiemost@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 
 #include "sidcxx11.h"
 
+#include <algorithm>
+#include <iterator>
 #include <cstring>
 #include <stdint.h>
 
@@ -40,25 +42,29 @@ private:
     uint8_t lastpoke[0x20];
 
 protected:
-    virtual ~c64sid() {}
+    virtual ~c64sid() = default;
 
     virtual uint8_t read(uint_least8_t addr) = 0;
-    virtual void write(uint_least8_t addr, uint8_t data) = 0;
+    virtual void writeReg(uint_least8_t addr, uint8_t data) = 0;
 
 public:
     virtual void reset(uint8_t volume) = 0;
 
-    void reset() { memset(lastpoke, 0, 0x20); reset(0); }
+    void reset()
+    {
+        std::fill(std::begin(lastpoke), std::end(lastpoke), 0);
+        reset(0);
+    }
 
     // Bank functions
     void poke(uint_least16_t address, uint8_t value) override
     {
         lastpoke[address & 0x1f] = value;
-        write(address & 0x1f, value);
+        writeReg(address & 0x1f, value);
     }
     uint8_t peek(uint_least16_t address) override { return read(address & 0x1f); }
 
-    void getStatus(uint8_t regs[0x20]) const { memcpy(regs, lastpoke, 0x20); }
+    void getStatus(uint8_t regs[0x20]) const { std::memcpy(regs, lastpoke, 0x20); }
 };
 
 }
