@@ -86,13 +86,13 @@ const char CHR_tab[256] =
 };
 
 /// The Commodore 64 memory size
-const uint_least32_t MAX_MEMORY = 65536;
+constexpr uint_least32_t MAX_MEMORY = 65536;
 
 /// C64KB + LOAD + PSID
-const uint_least32_t MAX_FILELEN = MAX_MEMORY + 2 + 0x7C;
+constexpr uint_least32_t MAX_FILELEN = MAX_MEMORY + 2 + 0x7C;
 
 /// Minimum load address for real c64 only tunes
-const uint_least16_t SIDTUNE_R64_MIN_LOAD_ADDR = 0x07e8;
+constexpr uint_least16_t SIDTUNE_R64_MIN_LOAD_ADDR = 0x07e8;
 
 SidTuneBase* SidTuneBase::load(const char* fileName, const char **fileNameExt,
                  bool separatorIsSlash)
